#include <avr/io.h>
#include <avr/interrupt.h>

#include "UART.h"
#include "RGB.h"
#include "Button.h"
#include "CapacitorDisconnector.h"
#include "HighResolutionTimer.h"
#include "PathSelector.h"

#define CHARGE_TIME_SEC				5
#define COMPARATOR_LEVEL			255

#define STATE_READY					1
#define STATE_CHARGING				2
#define STATE_CHARGED				3
#define STATE_DISCHARGING			4
#define STATE_DISCHARGED			5
#define STATE_COMPLETED				6

int state = STATE_READY;

void Clock_Init() {
	CCP = CCP_IOREG_gc;
	CLKCTRL.XOSCHFCTRLA = CLKCTRL_FRQRANGE_16M_gc | CLKCTRL_ENABLE_bm;
	CCP = CCP_IOREG_gc;
	CLKCTRL.XOSC32KCTRLA = CLKCTRL_ENABLE_bm;
	CCP = CCP_IOREG_gc;
	CLKCTRL.MCLKCTRLB = CLKCTRL_PDIV_4X_gc | CLKCTRL_PEN_bm;
	CCP = CCP_IOREG_gc;
	CLKCTRL.MCLKCTRLA = CLKCTRL_CLKSEL_EXTCLK_gc;
	
	for (volatile int i = 0; i < 1000; i++) {}
}

int main(void) {
	// debug
	PORTD.DIRSET = PIN2_bm;
	PORTD.OUTSET = PIN2_bm;
	
	Clock_Init();
	Comparator_Init();
	RGB_Init();
	Button_Init();
	PathSelector_Init();
	CapacitorDisconnector_Init();
	HighResolutionTimer_Init();
	
	// enable interrupts
	sei();
	
	UART_Init();
	UART_PrintString("Init OK\r\n");
	
	RGB_SetColor(0, 255, 0);
	
	while (1) {
		if (state == STATE_READY) {
			if (Button_IsPressed()) {
				RtcTimer_ConfigureAndStart(CHARGE_TIME_SEC);
				PathSelector_SelectCharger();
				CapacitorDisconnector_Connect();
				
				RGB_SetColor(255, 255, 0);
				UART_PrintString("Charging started.\r\n");
				
				state = STATE_CHARGING;
			}
		} else if (state == STATE_CHARGING) {
			if (RtcTimer_IsHit()) {
				state = STATE_CHARGED;
			}
		} else if (state == STATE_CHARGED) {
			CapacitorDisconnector_DriveByComparator(COMPARATOR_LEVEL);
			PathSelector_SelectDischarger();
			HighResolutionTimer_Start();
			
			RGB_SetColor(0, 0, 255);
			UART_PrintString("Charging completed.\r\nDischarging.\r\n");
			
			state = STATE_DISCHARGING;
		} else if (state == STATE_DISCHARGING) {
			for (volatile uint32_t i = 0; i < 30000 && HighResolutionTimer_IsRunning(); i++) {}
			RGB_ToggleBlue();
			
			if (!HighResolutionTimer_IsRunning()) {
				state = STATE_DISCHARGED;
			}
		} else if (state == STATE_DISCHARGED) {
			int comparatorState = Comparator_IsAbove();
			
			PathSelector_SelectCharger();
			CapacitorDisconnector_Disconnect();
			
			
			char buffer[16];
			snprintf(buffer, sizeof(buffer), "%ld", HighResolutionTimer_GetTime());
			UART_PrintString("Discharging completed.\r\nCharge duration: ");
			UART_PrintString(buffer);
			UART_PrintString("\r\n");
			
			if (HighResolutionTimer_GetOverflowCount()) {
				snprintf(buffer, sizeof(buffer), "%ld", HighResolutionTimer_GetOverflowCount());
				UART_PrintString("Timer overflown. Overflow count: ");
				UART_PrintString(buffer);
				UART_PrintString("\r\n");
				
				RGB_SetColor(255, 0, 0);
			} else {
				if (comparatorState) {
					RGB_SetColor(255, 0, 255);
				} else {
					RGB_SetColor(0, 255, 255);
				}
			}
			
			state = STATE_COMPLETED;
		} else if (state == STATE_COMPLETED) {
			for (volatile uint32_t i = 0; i < 700000; i++) {}
				
			RGB_SetColor(0, 255, 0);
			
			state = STATE_READY;
		}
		
	}
	
}

